﻿// 2011, Omer Faruk ZORLU
// http://www.omerfarukzorlu.com
using System.Linq;
using OMR.TableDataSync.Console.Configuration;
using OMR.TableDataSync.Console.Helpers;
using OMR.TableDataSync.Core.DataAccess;
using OMR.TableDataSync.Core.Domain;

namespace OMR.TableDataSync.Core
{
    public sealed class SyncManager
    {
        public static void SyncAllDatabases(TableDataSyncConfiguration config)
        {
            // // Create new DiscoverBLL instance for source server
            DiscoverBLL sourceDiscoverBLL = new DiscoverBLL(config.SourceServer.ConnectionString);

            DiscoverBLL destinationDiscoverBLL;

            // Create new TransferBLL instance
            TransferBLL transferBLL = new TransferBLL(config.SourceServer.ConnectionString);

            TableHash dtHashResult; // Holds destination table hash

            foreach (var table in config.TableConfigurations)
            {
                TraceHelper.Write("{0} sync process begins. Computing row hashes.\n", 5, table.Name);

                // Get source table hash
                var stHashResult = sourceDiscoverBLL.GetTableHash(table);

                // for each destination server
                foreach (var destination in config.DestinationServers)
                {
                    TraceHelper.Write("\n\n");
                    TraceHelper.Write("{0} in progress on [{1}]\n", 7, table.Name, destination.Name);

                    // Reset: Set all hash targets to not computed(-1)
                    // TODO: performance?
                    stHashResult.RowHashes.ToList().ForEach(k => k.Target = TableHashTargets.NotComputed);

                    // Create new DiscoverBLL instance for destination server
                    destinationDiscoverBLL = new DiscoverBLL(destination.ConnectionString);

                    // Get destination table hash
                    TraceHelper.Write("{0} computing row hashes.\n", 7, table.Name);
                    dtHashResult = destinationDiscoverBLL.GetTableHash(table);

                    // Compute equal, insert, update and delete targets
                    TraceHelper.Write("{0} computing data targets...\n", 7, table.Name);
                    DiscoverBLL.ComputeTargets(stHashResult, dtHashResult);

                    Summerize(stHashResult, dtHashResult); // TODO: 

                    if (0 < stHashResult.RowHashes.Where(k => k.Target == TableHashTargets.WillBeInserted).Count())
                        transferBLL.InsertRows(table, stHashResult, destination.ConnectionString);

                    if (0 < stHashResult.RowHashes.Where(k => k.Target == TableHashTargets.WillBeUpdated).Count())
                        transferBLL.UpdateRows(table, stHashResult, destination.ConnectionString);

                    if (0 < dtHashResult.RowHashes.Where(k => k.Target == TableHashTargets.WillBeDeleted).Count())
                        transferBLL.DeleteRows(table, dtHashResult, destination.ConnectionString);
                }

                TraceHelper.Write("{0} sync process ends\n", 8, table.Name);
            }

            TraceHelper.Write("Sync complated!\n", 11);
        }

        private static void Summerize(TableHash source, TableHash destination)
        {
            TraceHelper.Write("*************************\n");
            TraceHelper.Write("\tSyncronized:\t\t{0}\n", 10, source.RowHashes.Where(k => k.Target == TableHashTargets.Equal).Count());
            TraceHelper.Write("\tWill be inserted:\t{0}\n", 10, source.RowHashes.Where(k => k.Target == TableHashTargets.WillBeInserted).Count());
            TraceHelper.Write("\tWill be updated:\t{0}\n", 10, source.RowHashes.Where(k => k.Target == TableHashTargets.WillBeUpdated).Count());
            TraceHelper.Write("\tWill be deleted:\t{0}\n", 10, destination.RowHashes.Where(k => k.Target == TableHashTargets.WillBeDeleted).Count());
            TraceHelper.Write("*************************\n");
        }
    }
}
